import java.awt.*;
import java.util.Vector;

class Board implements GameConstants {

  static final int
    B = BORDER, V = VACANT, P = PERSON, J = PLAYER;
  int[][] field = 
  {
  {B, B, B, B, B, B, B, B, B, B},
  {B, V, V, V, V, V, V, V, V, B}, 
  {B, V, V, V, V, V, V, V, V, B}, 
  {B, V, V, V, V, V, V, V, V, B}, 
  {B, V, V, V, P, J, V, V, V, B}, 
  {B, V, V, V, J, P, V, V, V, B}, 
  {B, V, V, V, V, V, V, V, V, B}, 
  {B, V, V, V, V, V, V, V, V, B}, 
  {B, V, V, V, V, V, V, V, V, B}, 
  {B, B, B, B, B, B, B, B, B, B}
  };

  // Possible moves for each participant
  private Vector[] moves = {new Vector(10), new Vector(10)};

  // The move that led to this Board
  Point previousMove = NULLMOVE;

  public Board () {
    computePossibleMoves(PERSON);
    computePossibleMoves(PLAYER);
  }

  // Clone this Board
  public Board duplicate () {
    Board b = new Board();
    for (int col = 0; col < 10; col++)
      for (int row = 0; row < 10; row++)
        b.field[row][col] = field[row][col];
    b.moves[0] = (Vector)moves[0].clone();
    b.moves[1] = (Vector)moves[1].clone();
    b.previousMove = new Point(previousMove);
    return b;
  }

  // Tell the legal moves for a participant
  public Vector tellMoves (int participant) {
    return (Vector)moves[participant].clone();
  }

  // Identify the opposing participant
  public int opposite (int participant) {
    if (participant == PERSON) return PLAYER;
    else                       return PERSON;
  }

  // Tell if a move is legal
  public boolean isLegal (int participant, Point move) {
    return moves[participant].contains(move);
  }

  // Count the number of stones for a participant
  public int count (int participant) {
    int n = 0;
    for (int i = 1; i < 9; i++) {
      for (int j = 1; j < 9; j++) {
        if ( field[i][j] == participant) n++;
      }
    }
  return n;
  }

  // Tell how many legal moves a participant may choose from.
  public int howManyMoves (int participant) {
    return moves[participant].size();
  }

  // Tell what's on a square of this Board
  public int squareStatus (int row, int col) {
    return field[row][col];
  }

  // Movement deltas for NW,N,NE,W,E,SW,S,SE
  private static Point[] spokes = 
    { new Point(-1,-1), new Point(0,-1), new Point(1,-1),
      new Point (-1,0),                  new Point(1,0),
      new Point (-1,1), new Point(0,1),  new Point(1,1)
    };

  // Determine the legal moves for a participant
  private void computePossibleMoves (int participant) {
    int op = opposite(participant);

    moves[participant].clear();
    for (int col = 1; col < 9; col++) { 
      for (int row = 1; row < 9; row++) {
        // For this to be a valid move, 
        // this square must be vacant ...
        if (field[row][col] == VACANT) {
          // ... and along some spoke ...
          boolean again = true;
          for (int spoke = 0; spoke < 8 && again; spoke++) {
            int rowStep = spokes[spoke].x; 
            int colStep = spokes[spoke].y;
            // ... the adjacent square must be opponent ...
            if (field[row+rowStep][col+colStep] == op) {
              // ... and subsequent must be opponent ...
              int colInc = col+colStep+colStep;
              int rowInc = row+rowStep+rowStep; 
              while (field[rowInc][colInc] == op) {
                colInc = colInc + colStep;
                rowInc = rowInc + rowStep;
              }
              // ... and then we must find our own.
              if (field[rowInc][colInc] == participant) {
                moves[participant].addElement(new Point(row, col));
                // No need to consider further spokes, since
                // this spoke validates this move.
                again = false;  
              }
            }
          }
        }
      }
    }
  }

  // Record a move on this Board
  public void makeMove (int participant, Point move) {

    previousMove = move;
    int op = opposite(participant);
    int row = move.x, col = move.y;
    field[row][col] = participant;

    // Consider all spokes around this move.
    for (int spoke = 0; spoke < 8; spoke++) {
      int colStep = spokes[spoke].y; 
      int rowStep = spokes[spoke].x;
      // Do we flip this spoke?  Only if it starts with
      // an opponent's stone ...
      if (field[row+rowStep][col+colStep] == op) {
        // ... and has more opponent's stones ...
        int colInc = col+colStep+colStep;
        int rowInc = row+rowStep+rowStep; 
        while (field[rowInc][colInc] == op) {
          colInc = colInc + colStep;
          rowInc = rowInc + rowStep;
        }
        // ... and ends with our stone.
        if (field[rowInc][colInc] == participant) {
          // Flip the continuous line of opponent's 
          // stones on this spoke.
          colInc = col+colStep;
          rowInc = row+rowStep; 
          while (field[rowInc][colInc] == op) {
            field[rowInc][colInc] = participant;
            colInc = colInc + colStep;
            rowInc = rowInc + rowStep;
          }
        }
      }
    }
    computePossibleMoves(participant);
    computePossibleMoves(op);
  }
}
